/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_ATTRIBUTE_H_
#define IDLC_ATTRIBUTE_H_

#include <string>
#include <list>
#include <memory>

namespace tidl {

class Attribute {
 public:
  Attribute(std::string key, std::string value, unsigned line);

  const std::string& GetKey() const;
  const std::string& GetValue() const;
  const unsigned GetLine() const;

 private:
  std::string key_;
  std::string value_;
  unsigned line_;
};

class Attributes {
 public:
  void Add(Attribute* attr);
  const std::list<std::unique_ptr<Attribute>>& GetAttrs() const;
  bool Exist(Attribute* attr) const;

 private:
  std::list<std::unique_ptr<Attribute>> attrs_;
};

}  // namespace tidl

#endif  // IDLC_ATTRIBUTE_H_
