/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CS_GEN_CS_CB_RPC_PORT_H_
#define IDLC_CS_GEN_CS_CB_RPC_PORT_H_

const char cs_cb_rpc_port[] =
R"__cs_cb(
namespace Tizen.Applications.RPCPort
{
    public class InvalidIOException : InvalidOperationException { }
    public class InvalidIDException : InvalidOperationException { }
    public class PermissionDeniedException : InvalidOperationException { }
    public class InvalidProtocolException : InvalidOperationException { }
    public class NotConnectedSocketException : InvalidOperationException { }
    public class InvalidCallbackException : InvalidOperationException { }

    public class Port
    {
        internal IntPtr Handle { get; set; }
        internal Port() { }
    }

    public class Parcel : IDisposable
    {
        private IntPtr _handle;

        public Parcel()
        {
            Interop.LibRPCPort.Parcel.Create(out _handle);
        }

        public Parcel(Port port)
        {
            Interop.LibRPCPort.Parcel.CreateFromPort(out _handle, port.Handle);
        }

        public void Send(Port p)
        {
            Interop.LibRPCPort.Parcel.Send(_handle, p.Handle);
        }

        public void WriteByte(byte b)
        {
            Interop.LibRPCPort.Parcel.WriteByte(_handle, b);
        }

        public void WriteShort(short b)
        {
            Interop.LibRPCPort.Parcel.WriteInt16(_handle, b);
        }

        public void WriteInt(int b)
        {
            Interop.LibRPCPort.Parcel.WriteInt32(_handle, b);
        }

        public void WriteLong(long b)
        {
            Interop.LibRPCPort.Parcel.WriteInt64(_handle, b);
        }

        public void WriteFloat(float b)
        {
            Interop.LibRPCPort.Parcel.WriteFloat(_handle, b);
        }

        public void WriteDouble(double b)
        {
            Interop.LibRPCPort.Parcel.WriteDouble(_handle, b);
        }

        public void WriteString(string b)
        {
            Interop.LibRPCPort.Parcel.WriteString(_handle, b);
        }

        public void WriteBool(bool b)
        {
            Interop.LibRPCPort.Parcel.WriteBool(_handle, b);
        }

        public void WriteBundle(Bundle b)
        {
            Interop.LibRPCPort.Parcel.WriteBundle(_handle, b.SafeBundleHandle.DangerousGetHandle());
        }

        public void WriteArrayCount(int cnt)
        {
            Interop.LibRPCPort.Parcel.WriteArrayCount(_handle, cnt);
        }

        public byte ReadByte()
        {
            Interop.LibRPCPort.Parcel.ReadByte(_handle, out byte b);
            return b;
        }

        public short ReadShort()
        {
            Interop.LibRPCPort.Parcel.ReadInt16(_handle, out short b);
            return b;
        }

        public int ReadInt()
        {
            Interop.LibRPCPort.Parcel.ReadInt32(_handle, out int b);
            return b;
        }

        public long ReadLong()
        {
            Interop.LibRPCPort.Parcel.ReadInt64(_handle, out long b);
            return b;
        }

        public float ReadFloat()
        {
            Interop.LibRPCPort.Parcel.ReadFloat(_handle, out float b);
            return b;
        }

        public double ReadDouble()
        {
            Interop.LibRPCPort.Parcel.ReadDouble(_handle, out double b);
            return b;
        }

        public string ReadString()
        {
            Interop.LibRPCPort.Parcel.ReadString(_handle, out string b);
            return b;
        }

        public bool ReadBool()
        {
            Interop.LibRPCPort.Parcel.ReadBool(_handle, out bool b);
            return b;
        }

        public Bundle ReadBundle()
        {
            Interop.LibRPCPort.Parcel.ReadBundle(_handle, out IntPtr b);

            return new Bundle(new SafeBundleHandle(b, true));
        }

        public int ReadArrayCount()
        {
            Interop.LibRPCPort.Parcel.ReadArrayCount(_handle, out int b);
            return b;
        }

        public void Write(byte[] bytes)
        {
            Interop.LibRPCPort.Parcel.Write(_handle, bytes, bytes.Length);
        }

        public byte[] Read(int size)
        {
            var ret = new byte[size];
            Interop.LibRPCPort.Parcel.Read(_handle, ret, size);
            return ret;
        }

        #region IDisposable Support
        private bool disposedValue = false;

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                }

                if (_handle != IntPtr.Zero)
                {
                    Interop.LibRPCPort.Parcel.Destroy(_handle);
                    _handle = IntPtr.Zero;
                }

                disposedValue = true;
            }
        }

        ~Parcel()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion
    }

    public abstract class ProxyBase : IDisposable
    {
        private Interop.LibRPCPort.Proxy.ConnectedEventCallback _connectedEventCallback;
        private Interop.LibRPCPort.Proxy.DisconnectedEventCallback _disconnectedEventCallback;
        private Interop.LibRPCPort.Proxy.RejectedEventCallback _rejectedEventCallback;
        private Interop.LibRPCPort.Proxy.ReceivedEventCallback _receivedEventCallback;
        private IntPtr _proxy;

        protected Port Port { get; private set; }

        public ProxyBase()
        {
            if (Interop.LibRPCPort.Proxy.Create(out _proxy) != Interop.LibRPCPort.ErrorCode.None)
                throw new InvalidIOException();
            _connectedEventCallback = new Interop.LibRPCPort.Proxy.ConnectedEventCallback(OnConnectedEvent);
            _disconnectedEventCallback = new Interop.LibRPCPort.Proxy.DisconnectedEventCallback(OnDisconnectedEvent);
            _rejectedEventCallback = new Interop.LibRPCPort.Proxy.RejectedEventCallback(OnRejectedEvent);
            _receivedEventCallback = new Interop.LibRPCPort.Proxy.ReceivedEventCallback(OnReceivedEvent);
            Interop.LibRPCPort.Proxy.AddConnectedEventCb(_proxy, _connectedEventCallback, IntPtr.Zero);
            Interop.LibRPCPort.Proxy.AddDisconnectedEventCb(_proxy, _disconnectedEventCallback, IntPtr.Zero);
            Interop.LibRPCPort.Proxy.AddRejectedEventCb(_proxy, _rejectedEventCallback, IntPtr.Zero);
            Interop.LibRPCPort.Proxy.AddReceivedEventCb(_proxy, _receivedEventCallback, IntPtr.Zero);
        }

        protected void Connect(string appid, string port)
        {
            var err = Interop.LibRPCPort.Proxy.Connect(_proxy, appid, port);
            switch (err)
            {
                case Interop.LibRPCPort.ErrorCode.InvalidParameter:
                    throw new InvalidIDException();
                case Interop.LibRPCPort.ErrorCode.PermissionDenied:
                    throw new PermissionDeniedException();
                case Interop.LibRPCPort.ErrorCode.IoError:
                    throw new InvalidIOException();
            }
        }

        protected abstract void OnConnectedEvent(string endPoint, string portName, Port port);
        protected abstract void OnDisconnectedEvent(string endPoint, string portName);
        protected abstract void OnReceivedEvent(string endPoint, string portName);
        protected abstract void OnRejectedEvent(string endPoint, string portName);

        private void OnConnectedEvent(string endPoint, string portName, IntPtr port, IntPtr data)
        {
            Port = new Port() { Handle = port };
            OnConnectedEvent(endPoint, portName, Port);
        }

        private void OnDisconnectedEvent(string endPoint, string portName, IntPtr data)
        {
            Port = null;
            OnDisconnectedEvent(endPoint, portName);
        }

        private void OnReceivedEvent(string endPoint, string portName, IntPtr data)
        {
            OnReceivedEvent(endPoint, portName);
        }

        private void OnRejectedEvent(string endPoint, string portName, IntPtr data)
        {
            OnRejectedEvent(endPoint, portName);
        }

        #region IDisposable Support
        private bool disposedValue = false;

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                }
                if (_proxy != IntPtr.Zero)
                    Interop.LibRPCPort.Proxy.Destroy(_proxy);
                _proxy = IntPtr.Zero;

                disposedValue = true;
            }
        }

        ~ProxyBase()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion
    }

    public abstract class StubBase : IDisposable
    {
        private Interop.LibRPCPort.Stub.ConnectedEventCallback _connectedEventCallback;
        private Interop.LibRPCPort.Stub.DisconnectedEventCallback _disconnectedEventCallback;
        private Interop.LibRPCPort.Stub.ReceivedEventCallback _receivedEventCallback;
        private IntPtr _stub;

        public string PortName { get; private set; }

        public StubBase(string portName)
        {
            if (Interop.LibRPCPort.Stub.Create(out _stub, portName) != Interop.LibRPCPort.ErrorCode.None)
                throw new InvalidIOException();
            PortName = portName;
            _connectedEventCallback = new Interop.LibRPCPort.Stub.ConnectedEventCallback(OnConnectedEvent);
            _disconnectedEventCallback = new Interop.LibRPCPort.Stub.DisconnectedEventCallback(OnDisconnectedEvent);
            _receivedEventCallback = new Interop.LibRPCPort.Stub.ReceivedEventCallback(OnReceivedEvent);
            Interop.LibRPCPort.Stub.AddReceivedEventCb(_stub, _receivedEventCallback, IntPtr.Zero);
            Interop.LibRPCPort.Stub.AddConnectedEventCb(_stub, _connectedEventCallback, IntPtr.Zero);
            Interop.LibRPCPort.Stub.AddDisconnectedEventCb(_stub, _disconnectedEventCallback, IntPtr.Zero);
        }

        protected void Listen()
        {
            var err = Interop.LibRPCPort.Stub.Listen(_stub);
            switch (err)
            {
                case Interop.LibRPCPort.ErrorCode.InvalidParameter:
                case Interop.LibRPCPort.ErrorCode.IoError:
                    throw new InvalidIOException();
            }
        }

        protected void AddPrivilege(string privilege)
        {
            var err = Interop.LibRPCPort.Stub.AddPrivilege(_stub, privilege);
            switch (err)
            {
                case Interop.LibRPCPort.ErrorCode.InvalidParameter:
                    throw new InvalidIOException();
            }
        }

        protected void SetTrusted(bool trusted)
        {
            var err = Interop.LibRPCPort.Stub.SetTrusted(_stub, trusted);
            switch (err)
            {
                case Interop.LibRPCPort.ErrorCode.InvalidParameter:
                    throw new InvalidIOException();
            }
        }

        protected abstract void OnConnectedEvent(string sender, string instance);

        protected abstract void OnDisconnectedEvent(string sender, string instance);

        protected abstract bool OnReceivedEvent(string sender, string instance, Port port);

        protected abstract void OnTerminatedEvent();

        private void OnConnectedEvent(string sender, string instance, IntPtr data)
        {
            OnConnectedEvent(sender, instance);
        }

        private void OnDisconnectedEvent(string sender, string instance, IntPtr data)
        {
            OnDisconnectedEvent(sender, instance);
        }

        private int OnReceivedEvent(string sender, string instance, IntPtr port, IntPtr data)
        {
            bool b = OnReceivedEvent(sender, instance, new Port() { Handle = port });
            if (b)
                return 0;
            return -1;
        }

        #region IDisposable Support
        private bool disposedValue = false;

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                }

                OnTerminatedEvent();

                if (_stub != IntPtr.Zero)
                    Interop.LibRPCPort.Proxy.Destroy(_stub);
                _stub = IntPtr.Zero;

                disposedValue = true;
            }
        }

        ~StubBase()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion
    }
}

)__cs_cb";

#endif  // IDLC_CS_GEN_CS_CB_RPC_PORT_H_
