/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/attribute.h"

class AttributeTest : public testing::Test {
 public:
  virtual void SetUp() {}
  virtual void TearDown() {}
};

TEST_F(AttributeTest, Attribute_Constructor) {
  tidl::Attribute* attr = new tidl::Attribute("key", "value", __LINE__);
  EXPECT_NE(attr, nullptr);
  delete attr;
}

TEST_F(AttributeTest, Attribute_GetKey) {
  tidl::Attribute* attr = new tidl::Attribute("key", "value", __LINE__);
  EXPECT_NE(attr, nullptr);
  EXPECT_EQ(attr->GetKey(), "key");
  delete attr;
}

TEST_F(AttributeTest, Attribute_GetValue) {
  tidl::Attribute* attr = new tidl::Attribute("key", "value", __LINE__);
  EXPECT_NE(attr, nullptr);
  EXPECT_EQ(attr->GetValue(), "value");
  delete attr;
}

TEST_F(AttributeTest, Attribute_GetLine) {
  unsigned line = __LINE__;
  tidl::Attribute* attr = new tidl::Attribute("key", "value", line);
  EXPECT_NE(attr, nullptr);
  EXPECT_EQ(attr->GetLine(), line);
  delete attr;
}

class AttributesTest : public testing::Test {
 public:
  virtual void SetUp() {}
  virtual void TearDown() {}
};

TEST_F(AttributesTest, Attributes_Constructor) {
  tidl::Attributes* attrs = new tidl::Attributes();
  EXPECT_NE(attrs, nullptr);
  delete attrs;
}

TEST_F(AttributesTest, Attributes_Add) {
  tidl::Attributes* attrs = new tidl::Attributes();
  EXPECT_NE(attrs, nullptr);
  attrs->Add(new tidl::Attribute("key", "value", __LINE__));

  bool flag = false;
  for (auto& attr : attrs->GetAttrs()) {
    if (attr->GetKey() == "key" &&
        attr->GetValue() == "value")
      flag = true;
  }
  EXPECT_EQ(flag, true);
  delete attrs;
}

TEST_F(AttributesTest, Attributes_GetAttrs) {
  tidl::Attributes* attrs = new tidl::Attributes();
  EXPECT_NE(attrs, nullptr);
  attrs->Add(new tidl::Attribute("key1", "value1", __LINE__));
  attrs->Add(new tidl::Attribute("key2", "value2", __LINE__));

  int count = 0;
  for (auto& attr : attrs->GetAttrs()) {
    if (attr->GetKey() == "key1" &&
        attr->GetValue() == "value1")
      count++;
    if (attr->GetKey() == "key2" &&
        attr->GetValue() == "value2")
      count++;
  }
  EXPECT_EQ(count, 2);
  delete attrs;
}

TEST_F(AttributesTest, Attributes_Exist) {
  tidl::Attributes* attrs = new tidl::Attributes();
  EXPECT_NE(attrs, nullptr);
  attrs->Add(new tidl::Attribute("key", "value", __LINE__));
  tidl::Attribute* attr = new tidl::Attribute("key", "value", __LINE__);
  EXPECT_NE(attr, nullptr);
  EXPECT_EQ(attrs->Exist(attr), true);
  delete attr;
  delete attrs;
}
